{% extends 'watson/base.html' %}

{% block body %}
<p>Hi,</p>
<p>We've received a request to reset your password.</p>
<p><a href="{{ url(config()['auth']['reset_password']['urls']['route'], scheme=request.url.scheme, host=request.url.hostname, port=(request.url.port if request.url.port != 80 else None)) }}?token={{ token.token }}">Reset Password</a></p>
<p>If you did not make this request, you can ignore this email and your password will not be changed.</p>
{% endblock %}
